/* Copyright (c) Oracle Corporation 2000.  All Rights Reserved. */

package oracle.forms.demos;

import java.awt.*;
import java.awt.event.*;
import java.net.MalformedURLException;
import java.net.URL;

import oracle.ewt.button.PushButton;
import oracle.forms.handler.IHandler;
import oracle.forms.ui.IView;
import oracle.forms.ui.VButton;
import oracle.forms.properties.ID;
import oracle.forms.properties.Property;

/**
 * <H3><FONT COLOR="003399">Oracle Forms Server 6i PJC Example</FONT></H3>
 *
 * This is a PJC that implements a web style <B>Rollover image button</b>
 * where the image changes when the user moves the mouse over the component.  The standard Forms
 * <code>oracle.forms.ui.VButton</code> is subclassed so only the additional functionality is required
 * to be added.
 *
 * We register two new forms properties, IMAGE_NAME_ON and IMAGE_NAME_OFF.  These properties form the
 * the URL of the image files that the button will display.  These properties will be set by PL/SQL
 * trigger code.
 *
 * @version 1.0 09/13/1999 created<br>
 * @version 1.1 12/09/1999 modified to support 6i features to allow dynamic custom property manipulation
 * @version 1.2 02/20/2000 removed 6.0 stuff and renamed package to oracle.forms.demos for distribution
 * @author Steve Button (sbutton@oracle.com)
 */

public class RolloverButton
       extends VButton
{
  /**
   * the property registered to specify the on image to be used
   */
  public final static ID IMAGE_NAME_ON = ID.registerProperty("IMAGE_NAME_ON");
  /**
   * the property registered to specify the off image to be used
   */
  public final static ID IMAGE_NAME_OFF = ID.registerProperty("IMAGE_NAME_OFF");
  /**
   * the name of the on image
   */
  private String  mImageNameOn;
  /**
   * the name of the off image
   */
  private String  mImageNameOff;
  /**
   * storage for the handler for this class
   */
  private IHandler  mHandler;
  /**
   * the current state ON | OFF
   */
  private int STATE;
  /**
   * define ON
   */
  private final int ON = 1;
  /**
   * define OFF
   */
  private final int OFF = 0;
  /**
   * array to hold the images used to represent the state of the button
   */
  private Image[] mImages = { null, null };
  /**
   * the classname used for debugging purposes
   */
  private final String CLASSNAME =getClass().getName();
  /**
   * do we want to debug for this class?  Set this to true to see debug messages.
   */
  private final boolean DEBUG = false;
  /**
   * Calls the constructor for the image rollover button and sets the button borders to be rounded.
   */
  public RolloverButton()
  {
    super();
    Utils.setDebug(CLASSNAME,DEBUG);
    addMouseListener(new RolloverButtonMouseAdapter());
    setLeftmost(true);
    setRightmost(true);
  }

  /**
   * Implementation of IView interface which provides an initialization opportunity for the component
   *
   * @param handler - message handler associated with this view.
   * @see oracle.forms.ui.IView
   */
  public void init(IHandler handler)
  {
    mHandler = handler;
    super.init(handler);

    //
    // These are the defaults set so that the developer knows the button is working.  These
    // can be overridden with the set_custom_property(hBean,1,'IMAGE_NAME_ON',imagename) built-in
    //
    setProperty(IMAGE_NAME_OFF, "off.gif");
    setProperty(IMAGE_NAME_ON,  "on.gif");
    setImage(OFF);
  }

  /**
   * Implementation of IView interface which returns the value of a requested property
   *
   * @param pid the property id that represents the property to be set
   * @return the value of the property id
   * @see oracle.forms.ui.IView
   */
  public Object getProperty(ID pid)
  {
    //
    // ImageButton Properties
    //
    if ( pid == IMAGE_NAME_OFF )
    {
      return mImageNameOff;
    }
    if ( pid == IMAGE_NAME_ON )
    {
      return mImageNameOn;
    }

    return super.getProperty(pid);
  }


  /**
   * Implementation of IView interface which sets a requested property to a given value
   *
   * @param id    property to be set.
   * @param value value of the property id.
   * @return      true if the property could be set, false otherwise.
   * @see oracle.forms.ui.IView
   */
  public boolean setProperty(ID pid, Object value)
  {
    boolean success = true;
    Utils.debug(CLASSNAME,"setProperty");
    //
    // ImageButton Properties
    //
    if ( pid == IMAGE_NAME_OFF )
    {
      Utils.debug(CLASSNAME,"setProperty: IMAGE_NAME_OFF - " + value.toString());
      mImageNameOff = (String) value;
      loadImage(OFF,mImageNameOff);

      // reset the currrently drawn image if needed
      setImage(OFF,STATE);
      return true;
     }

     if ( pid == IMAGE_NAME_ON )
     {
       Utils.debug(CLASSNAME,"setProperty: IMAGE_NAME_ON - " + value.toString());
       mImageNameOn = (String)value;
       loadImage(ON, mImageNameOn);

       // reset the currently drawn image if needed
        setImage(ON,STATE);
        return true;
     }
     // let the VButton class handle the other properties
    return super.setProperty(pid, value);
  }

  /**
   * Set the image displayed to the requested image if the requested image is the current image.
   * Effectively does a redraw of the currently displayed image if the image is changed by the user.
   * @param which the image state to draw, value values ON | OFF
   * @param current the current state being represented by the image
   */
  private void setImage(int which, int current)
  {
    if(which==current)
      setImage(which);
  }

  /**
   * Set the image displayed to the appropriate image depending on the user action.
   *
   * @param which the image to display, valid values ON and OFF
   */
  private void setImage(int which)
  {
    //
    // set the state to the on/off status sent here
    //
    STATE=which;
    if(which==ON)
      Utils.debug(CLASSNAME,"setImage ON");
    else
      Utils.debug(CLASSNAME,"setImage OFF");
    this.setImage(mImages[which]);
  }

  /**
   * Load the image specified as a string value and associate it with the identified state
   *
   * @param which the state to associate the image with
   * @param value the name of the image file to load
   */
  public void loadImage(int which, String value)
  {
    Utils.debug(CLASSNAME,"loadImage : value - " + value.toString());

    try
    {
      URL url = new URL(mHandler.getCodeBase(), value);
      mImages[which] = getToolkit().getImage(url);
    }
    catch(Exception ex)
    {
      Utils.debug(CLASSNAME,"ImageOn: Bad image name - " + ex);
      ex.printStackTrace();
    }
  }

  /**
   * Private class to handle user mouse actions and to switch images when the user moves the mouse
   * into and out of the button object.
   */
  class RolloverButtonMouseAdapter
  extends MouseAdapter
  {
    /**
     * User moved the mouse over the button, swap to the on image.
     */
    public void mouseEntered(MouseEvent me)
    {
      //Utils.debug(CLASSNAME, "mouseEntered : SHOW ON IMAGE");
      setImage(ON);
    }

    /**
     * User moved the mouse out of the button, swap to the off image.
     */
    public void mouseExited(MouseEvent me)
    {
      //Utils.debug(CLASSNAME, "mouseExited : SHOW OFF IMAGE");
      setImage(OFF);
    }
  }
}
