/* Copyright (c) Oracle Corporation 2000.  All Rights Reserved. */

package oracle.forms.demos;

import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * 
 * <H3><FONT COLOR="003399">Oracle Forms Server 6i PJC Example</FONT></H3>
 *
 * <P>This class provides a series of common methods that are used by different
 * demonstration PJCs.  It provides common functions such as debugging messages, type conversion
 * for common type mappings and the creation of URLs.</P>
 *
 * <P>This public methods in this class are static so the class does not need to be instantiated,
 * the methods can just be invoked on the class itself.</P>
 *
 * @version <b>1.05</b> 09/03/1999  added createURL method<br>
 *          <b>1.0</b>  08/18/1999  created
 *
 * @author Steve Button (sbutton@oracle.com)
 */

public class Utils
{
  /**
   * The name of this class.
   */
  private static final String CLASSNAME = "oracle.forms.demos.Utils";

  /**
   * Are we debugging this class itself?
   */
  private static final boolean DEBUG = true;

  /**
   * A hashtable containing the class and its debug state.
   */
  private static Hashtable debugStatus = new Hashtable(5);

  /**
   * Set the debugging status for a class identified as cls.
   *
   * @param cls the name of the class
   * @param value the debugging status
   */
  public static void setDebug(String cls, boolean value)
  {
    debugStatus.put(cls,new Boolean(value));      
  }
    
  /**
   * Check to see if the class specified in cls has enabled debugging.
   *
   * @param cls the name of the class
   * @return true if the class has debuggin enabled, false otherwise
   */
  private static boolean isDebugOn(String cls)
  {
    Boolean bool = (Boolean)debugStatus.get(cls);
    if(bool!=null)
      return bool.booleanValue();
    else
      return false;
  }
    
  /**
   * Writes a debug message to the screen in the format of 
   * <LI><code>[CLASSNAME] : debugging message</code></LI>
   * if the debug status for the class is enabled.  The debug messages will
   * be written if the class has the debug status set to true.
   *
   * @param cls the name of the class
   * @msg the debugging message to write out 
   */
  public static void debug(String cls, String msg)
  {
    if(isDebugOn(cls))
      System.out.println("[" + cls + "] : " + msg);
  } 

  /**
   * A debugging message mechanism used by this class only
   * 
   * @param msg the debugging message to write out
   */
  private static void debug(String msg)
  {
    if(DEBUG)
      System.out.println("[" + CLASSNAME + "] : " + msg);
  } 


  /**
   * Uses the default colors defined in Color to map a 
   * string value representation of a color to a Color object.
   * Defaults to returning Color.white if the color can't be mapped.
   *
   * @param       color the string representation of the color
   * @return      Color object that matches the color specified in the string
   */
  public static Color getColorFromString(String color)
  {
    return getColorFromString(color,Color.white);
  }

  /**
   * Uses the default colors defined in Color to map a 
   * string value representation of a color to a Color object.
   *
   * @param    color    the string representation of the color
   * @param    def      the color that should be returned if no match is found
   * @return   Color object that matches the color specified in the string
   */
  public static Color getColorFromString(String color,Color def)
  {
    if(color.equalsIgnoreCase("white"))
      return Color.white;
    else if(color.equalsIgnoreCase("lightgray"))
      return Color.lightGray;
    else if(color.equalsIgnoreCase("grey"))
      return Color.gray;
    else if(color.equalsIgnoreCase("darkgrey"))
      return Color.darkGray;
    else if(color.equalsIgnoreCase("black"))
      return Color.black;
    else if(color.equalsIgnoreCase("red"))
      return Color.red;
    else if(color.equalsIgnoreCase("pink"))
      return Color.pink;
    else if(color.equalsIgnoreCase("orange"))
      return Color.orange;
    else if(color.equalsIgnoreCase("yellow"))
      return Color.yellow;
    else if(color.equalsIgnoreCase("green"))
      return Color.green;
    else if(color.equalsIgnoreCase("magenta"))
      return Color.magenta;
    else if(color.equalsIgnoreCase("cyan"))
      return Color.cyan;
    else if(color.equalsIgnoreCase("blue"))
      return Color.blue;
    else
      return def;
  } 
  

  /**
   * Creates a and returns a color object from RGB values in a string.
   * Will return Color.red
   * in the case where the string is incorrect or a color could not be created
   * from the input string.
   *
   * @param    color    the string representation of the color
   * @param    def      the default color to return in case of a screwup
   * @return   Color object that matches the color specified in the string
   */
  public static Color getColorFromRGBString(String color)
  {
    return getColorFromRGBString(color,Color.red);
  }
  
  /**
   * Creates a color object from RGB values in a string
   * string value representation of a color to a Color object.
   *
   * @param    color    the string representation of the color
   * @param    def      the default color to return in case of a screwup
   * @return   Color object that matches the color specified in the string
   */
  public static Color getColorFromRGBString(String color, Color def)
  {
    StringTokenizer st;
    String[] colVals = { null, null, null };
    int[] colvals = { 0,0,0 };
    int colindex=0;
    
    
    // the options to pass in the color string are
    // r,g,b with the values of r,g,b being 0-255
    // r g b with the values of r,g,b being 0-255

    if(color.indexOf(",")!=-1)
    {
      // uses , delimiters
      debug(color + " => using comma delimeters");

      st = new StringTokenizer(color,",");

      while(st.hasMoreTokens())
      {
        colVals[colindex] = st.nextToken();
        colvals[colindex] = Integer.parseInt(colVals[colindex]);
        colindex++;
      }
      
      // did we get all the color vals from the string?  must be three
      if(colindex!= 2)
       return def;
      else
        return new Color(colvals[0],colvals[1],colvals[2]);
    }

    else if(color.indexOf(" ")!=-1)
    {
      // uses <space> delimiters
      debug(color + " => using space delimeters");
           
      st = new StringTokenizer(color," ");
      while(st.hasMoreTokens())
      {
        colVals[colindex] = st.nextToken();
        colvals[colindex] = Integer.parseInt(colVals[colindex]);
        colindex++;
      }
      
      // did we get all the color vals from the string?  must be three
      if(colindex!= 2)
       return def;
      else
        return new Color(colvals[0],colvals[1],colvals[2]);
    }
    else
    {
      // no delimeters that I understand, return the default color
      return def;
    }
  }

   
  /**
   * Create a URL based on a given resource name.  Firstly a network URL will attempt
   * to be created and failing that, a file based URL will be created.
   * 
   * @param fileName the resource to use for the URL
   * @return URL a URL that maps to the given resource
   */
  public static URL createURL(String fileName)
  {
    URL url = null;
    try
    {
      url = new URL(fileName);
    }
    catch (MalformedURLException ex)
    {
      File f = new File(fileName);

      try
      {
        String path = f.getAbsolutePath();
        String fs = System.getProperty("file.separator");

        if (fs.length() == 1)
        {
          char sep = fs.charAt(0);
          if (sep != '/')
            path = path.replace(sep, '/');
          if (path.charAt(0) != '/')
            path = '/' + path;
        }
        
        path = "file://" + path;
        url = new URL(path);
      }
      catch (MalformedURLException e)
      {
         System.out.println("Cannot create url for: " + fileName);
         System.exit(0);
      }
    }
    return url;
  }
 
}
